package com.june.entitytodbdoc.utils;

import com.june.entitytodbdoc.dto.DocumentInfoDTO;
import com.sun.javadoc.AnnotationDesc;
import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.FieldDoc;
import com.sun.javadoc.RootDoc;
import com.sun.tools.javadoc.Main;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 获取字段注释工具类
 *
 * @author june
 */
public class GetClassCommentUtils {

    /**
     * 会自动注入
     */
    private static RootDoc rootDoc;

    /**
     * 会自动调用这个方法
     *
     * @param root root
     * @return true
     */
    public static boolean start(RootDoc root) {
        rootDoc = root;
        return true;
    }

    /**
     * 生成文档
     *
     * @param beanFilePath 注意这里是.java文件绝对路径
     * @return 文档注释
     */
    public static Optional<DocumentInfoDTO> execute(String beanFilePath) {
        ClassDoc classDoc = getClassDoc(beanFilePath);
        // 获取类的信息
        DocumentInfoDTO documentMainInfo = getDocumentMainInfo(classDoc);
        if (Objects.isNull(documentMainInfo)) {
            return Optional.empty();
        }
        List<DocumentInfoDTO.FieldInfoDTO> fieldInfoDTOList = new LinkedList<>();
        // 处理父类字段
        handleSuperClassField(fieldInfoDTOList, classDoc);
        handleField(classDoc, fieldInfoDTOList);
        documentMainInfo.setFieldList(fieldInfoDTOList);
        // 处理类信息
        return Optional.of(documentMainInfo);
    }

    private static ClassDoc getClassDoc(String beanFilePath){
        if (beanFilePath.contains("$")) {
            return getInnerClassDoc(beanFilePath);
        }
        Main.execute(new String[]{"-doclet", GetClassCommentUtils.class.getName(), "-docletpath",
                Objects.requireNonNull(GetClassCommentUtils.class.getResource("/")).getPath(), "-encoding", "utf-8", beanFilePath});

        ClassDoc[] classes = rootDoc.classes();

        if (classes == null || classes.length == 0) {
            return null;
        }
        return classes[0];
    }

    private static final Pattern PATTERN = Pattern.compile("^(.+)\\$(.+?)\\.java$");

    private static ClassDoc getInnerClassDoc(String beanFilePath) {
        if (!beanFilePath.contains("$")) {
            // 判斷文件是否存在
            if (!new File(beanFilePath).exists()) {
                // 替换掉 最后 一个 \ 为 $
                int lastIndex = beanFilePath.lastIndexOf("\\");
                beanFilePath = beanFilePath.substring(0, lastIndex) + "$" + beanFilePath.substring(lastIndex + 1);
                return getInnerClassDoc(beanFilePath);
            }
            ClassDoc classDoc = getClassDoc(beanFilePath);
            return classDoc;
        }
        Matcher matcher = PATTERN.matcher(beanFilePath);
        if (!matcher.matches()) {
            return getClassDoc(beanFilePath);
        }
        String superClassPath = matcher.group(1) + ".java"; // 将 $ 前的路径替换为 .java 后缀
        String className = matcher.group(2); // 获取类名部分
        ClassDoc superClassDoc = getClassDoc(superClassPath);
        if (Objects.isNull(superClassDoc)) {
            return null;
        }
        ClassDoc[] classDocs = superClassDoc.innerClasses();
        if (ArrayUtils.isEmpty(classDocs)) {
            return null;
        }
        for (ClassDoc classDoc : classDocs) {
            if ((superClassDoc.typeName() + "." + className).equals(classDoc.name())) {
                return classDoc;
            }
        }
        return null;
    }

    private static void handleField(ClassDoc classDoc, List<DocumentInfoDTO.FieldInfoDTO> fieldInfoDTOList) {
        if (Objects.isNull(classDoc)) {
            return;
        }
        // 获取属性名称和注释
        FieldDoc[] fields = classDoc.fields(false);
        doHandleField(fieldInfoDTOList, fields, classDoc.qualifiedName());
    }

    private static void doHandleField(List<DocumentInfoDTO.FieldInfoDTO> fieldInfoDTOList, FieldDoc[] fields, String classFullName) {
        // 处理字段信息
        for (FieldDoc field : fields) {
            if (field.isTransient() || field.isFinal()) {
                continue;
            }
            DocumentInfoDTO.FieldInfoDTO fieldInfoDTO = new DocumentInfoDTO.FieldInfoDTO(field.name(), field.type().typeName(), getFirstLineComment(field.commentText()));
            // 判断是否是自己创建的类型
            String fieldClassFullName = field.type().qualifiedTypeName();
            String fieldClassTypeName = field.type().typeName();
            // 说明是子类
            if (fieldClassTypeName.contains(".") && !fieldClassFullName.contains("$")) {
                int lastIndex = fieldClassFullName.lastIndexOf(".");
                fieldClassFullName = fieldClassFullName.substring(0, lastIndex) + "$" + fieldClassFullName.substring(lastIndex + 1);
            }
            if (fieldClassFullName.endsWith("ArticleOptionInfo")) {
                fieldInfoDTO.setChildrenFieldList(getArticleOptionInfoFieldInfo());
            } else if (fieldClassFullName.contains("$")) {
                // 是否内部类
                String inJarClassPath = JarPathAndPackageMapping.getInJarClassPath(fieldClassFullName);
                if (StringUtils.isNotBlank(inJarClassPath)) {
                    ClassDoc innerClassDoc = getInnerClassDoc(inJarClassPath);
                    List<DocumentInfoDTO.FieldInfoDTO> children = new LinkedList<>();
                    handleField(innerClassDoc, children);
                    fieldInfoDTO.setChildrenFieldList(children);
                }
            } else if (isSelfClass(fieldClassFullName)) {
                List<DocumentInfoDTO.FieldInfoDTO> children = new LinkedList<>();
                handleField(getRealClassDoc(field.type().asClassDoc()), children);
                fieldInfoDTO.setChildrenFieldList(children);
            } else if (isCollection(fieldClassFullName)) {
                // 是否是集合，获取集合的真正名称
                String genericsType = ClassUtils.getGenericsType(classFullName, field.name());
                if (StringUtils.isNotBlank(genericsType)) {
                    // 设置当前字段的字段类型
                    fieldInfoDTO.setFieldType(String.format("%s&lt;%s&gt;", fieldInfoDTO.getFieldType(), genericsType.replace("$", ".").substring(genericsType.lastIndexOf(".") + 1)));
                    if (isSelfClass(genericsType)) {
                        getRealClassDoc(genericsType).ifPresent(fieldDoc -> {
                            List<DocumentInfoDTO.FieldInfoDTO> children = new LinkedList<>();
                            handleField(fieldDoc, children);
                            fieldInfoDTO.setChildrenFieldList(children);
                        });
                    }
                }
            }
            fieldInfoDTOList.add(fieldInfoDTO);
        }
    }

    private static boolean isSelfClass(String fieldClassFullName) {
        return fieldClassFullName.startsWith("com.jiankangyouyi");
    }

    /**
     * 是否是Collection的子类
     * @param fieldClassFullName 类全名
     * @return 是否是Collection的子类
     */
    private static boolean isCollection(String fieldClassFullName) {
        try {
            return Collection.class.isAssignableFrom(Class.forName(fieldClassFullName));
        } catch (ClassNotFoundException e) {
            return false;
        }
    }


    private static String getFirstLineComment(String comment) {
        String commentFinal = !comment.contains("\n")
                ? comment.replace("\n", "") : comment.substring(0, comment.indexOf("\n"));

        return replaceLink(commentFinal);
    }


    public static String replaceLink(String input) {
        if (input.contains("{@link")) {
            int index = input.indexOf("{@link");
            input = input.substring(0, index).trim();
        }
        return input;
    }

    public static void main(String[] args) {
        System.out.println(replaceLink("动作禁忌类型 {@link com.jiankangyouyi.cloud.basedata.api.enums.action.ActionFeatureEnum} "));
    }

    /**
     * 获取 BaseMongo 字段信息
     *
     * @return {@link List}<{@link DocumentInfoDTO.FieldInfoDTO}>
     */
    private static List<DocumentInfoDTO.FieldInfoDTO> getBaseMongoFieldInfo() {
        List<DocumentInfoDTO.FieldInfoDTO> resultList = new LinkedList<>();
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("_id", "ObjectId", "主键ID"));
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("validStatus", "String", "有效标识符,1-有效，2-无效"));
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("version", "Long", "乐观锁版本号"));
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("updateTime", "Date", "修改时间"));
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("createTime", "Date", "创建时间"));
        return resultList;
    }

    /**
     * 获取 ArticleOptionInfo 字段信息
     *
     * @return {@link List}<{@link DocumentInfoDTO.FieldInfoDTO}>
     */
    private static List<DocumentInfoDTO.FieldInfoDTO> getArticleOptionInfoFieldInfo() {
        List<DocumentInfoDTO.FieldInfoDTO> resultList = new LinkedList<>();
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("accountId", "String", "用户id"));
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("account", "String", "用户code"));
        resultList.add(new DocumentInfoDTO.FieldInfoDTO("accountName", "String", "操作这一刻的用户名称"));
        return resultList;
    }

    /**
     * 处理父类的字段信息
     *
     * @param fieldInfoDTOList 字段信息dto list
     * @param classDoc         类医生
     */
    private static void handleSuperClassField(List<DocumentInfoDTO.FieldInfoDTO> fieldInfoDTOList, ClassDoc classDoc) {
        ClassDoc superclassDoc = classDoc.superclass().asClassDoc();
        // 父类完整路径
        String superclassFullName = superclassDoc.qualifiedName();
        // 如果是 Object 直接return
        if (Object.class.getCanonicalName().equals(superclassFullName)) {
            return;
        }
        // 如果是 baseMongo 特殊处理一下
        if("com.jiankangyouyi.cloud.core.base.entity.BaseMongoEntity".equals(superclassFullName)){
            fieldInfoDTOList.addAll(getBaseMongoFieldInfo());
            return;
        }
        // 处理父类字段信息
        handleField(getRealClassDoc(superclassDoc), fieldInfoDTOList);
        // 递归处理父类
        handleSuperClassField(fieldInfoDTOList,superclassDoc);
    }

    /**
     * 获取 真正的 classDoc 有的在jar包中
     * @param classDoc  classDoc
     * @return classDoc
     */
    private static ClassDoc getRealClassDoc(ClassDoc classDoc){
        String classFullName = classDoc.qualifiedName();
        return getRealClassDoc(classFullName).orElse(classDoc);
    }


    /**
     * 获取 真正的 classDoc 有的在jar包中
     * @param classFullName  classFullName
     * @return classDoc
     */
    private static Optional<ClassDoc> getRealClassDoc(String classFullName){
        String inJarClassPath = JarPathAndPackageMapping.getInJarClassPath(classFullName);
        // 为空使用原来的
        if(StringUtils.isBlank(inJarClassPath)){
            return Optional.empty();
        }
        return Optional.ofNullable(getInnerClassDoc(inJarClassPath));
    }



    private static final List<String> DOCUMENT_AND_NAME = Arrays.asList(
            "org.springframework.data.mongodb.core.mapping.Document"
    );

    private static DocumentInfoDTO getDocumentMainInfo(ClassDoc classDoc) {
        if (Objects.isNull(classDoc)) {
            return null;
        }
        AnnotationDesc[] annotations = classDoc.annotations();
        // 注释为空 return
        if (ArrayUtils.isEmpty(annotations)) {
            return null;
        }
        Optional<String> documentNameOptional = Arrays.stream(annotations)
                .filter(ann -> DOCUMENT_AND_NAME.contains(ann.annotationType().qualifiedTypeName()))
                .map(ann -> ann.elementValues()[0].value().toString().replaceAll("\"", ""))
                .findFirst();
        return documentNameOptional.map(s -> new DocumentInfoDTO(s, getFirstLineComment(classDoc.commentText()))).orElse(null);
    }

}